@@ -21,8 +21,10 @@ import sys
 from optparse import OptionParser, OptionGroup
 import ConfigParser
 
+
 from txclib import utils, project
 from txclib.utils import parse_json, compile_json, relpath
+from txclib.config import OrderedRawConfigParser
 
 def cmd_init(argv, path_to_tx):
     "Initialize a new transifex project."
@@ -70,9 +72,9 @@ def cmd_init(argv, path_to_tx):
     # Handle the credentials through transifexrc
     home = os.path.expanduser("~")
     txrc = os.path.join(home, ".transifexrc")
-    config = ConfigParser.RawConfigParser()
+    config = OrderedRawConfigParser()
 
-    default_transifex = "http://www.transifex.net"
+    default_transifex = "https://www.transifex.net"
     transifex_host = options.host or raw_input("Transifex instance [%s]: " % default_transifex)
 
     if not transifex_host:
@@ -82,7 +84,7 @@ def cmd_init(argv, path_to_tx):
     if not os.path.exists(config_file):
         # The path to the config file (.tx/config)
         utils.MSG("Creating skeleton...")
-        config = ConfigParser.RawConfigParser()
+        config = OrderedRawConfigParser()
         config.add_section('main')
         config.set('main', 'host', transifex_host)
         # Touch the file if it doesn't exist
@@ -139,9 +141,6 @@ def cmd_set(argv, path_to_tx):
         " [doesn't work with the --auto-* commands].")
     group = OptionGroup(parser, "Extended options", "These options can only be"
         " used with the --auto-local command.")
-    group.add_option("-e","--regex", action="store_true", dest="regex",
-        default=False, help="Specify that expression is an extended regex"
-        " [requires --auto-local].")
     group.add_option("-s","--source-language", action="store",
         dest="source_language",
         default=None, help="Specify the source language of a resource"
@@ -170,10 +169,13 @@ def cmd_set(argv, path_to_tx):
             parser.error("Please specify a source language.")
         if not '<lang>' in expression:
             parser.error("The expression you have provided is not valid.")
+        if not utils.valid_slug(options.resource):
+            parser.error("Invalid resource slug. The format is <project_slug>"\
+                ".<resource_slug> and the valid characters include [_-\w].")
         _auto_local(path_to_tx, options.resource,
             source_language=options.source_language,
             expression = expression, source_file=options.source_file,
-            execute=options.execute, nosource=False, regex=options.regex)
+            execute=options.execute, nosource=False, regex=False)
     elif options.remote:
         try:
             url = args[0]
@@ -186,10 +188,6 @@ def cmd_set(argv, path_to_tx):
         if not resource:
             parser.error("You must specify a resource name with the"
                 " -r|--resource flag.")
-        try:
-            resource_slug = resource.split('.')[1]
-        except Exception,e:
-            parser.error("'%s' is not a valid resource slug." % resource)
 
         lang = options.language
         if not lang:
@@ -198,9 +196,9 @@ def cmd_set(argv, path_to_tx):
         if len(args) != 1:
             parser.error("Please specify a file.")
 
-        if not utils.valid_slug(resource_slug):
-            parser.error("Invalid characters in resource slug. Valid characters"
-                " include [_-\w].")
+        if not utils.valid_slug(resource):
+            parser.error("Invalid resource slug. The format is <project_slug>"\
+                ".<resource_slug> and the valid characters include [_-\w].")
 
         file = args[0]
         # Calculate relative path
@@ -223,8 +221,9 @@ def cmd_set(argv, path_to_tx):
         # Chdir to the root dir
         os.chdir(path_to_tx)
 
-        if not os.path.exists(path_to_file):
-            raise Exception("tx: File ( %s ) does not exist." % path_to_file)
+        if not utils.valid_slug(resource):
+            parser.error("Invalid resource slug. The format is <project_slug>"\
+                ".<resource_slug> and the valid characters include [_-\w].")
 
         _set_translation(path_to_tx, resource, lang, path_to_file)
 
@@ -237,18 +236,13 @@ def _auto_local(path_to_tx, resource, source_language, expression, execute=False
     """
     Auto configure local project
     """
-    expr_re = '.*%s.*' %  expression
-    if not regex:
-        # Force expr to be a valid regex expr (escaped) but keep <lang> intact
-        expr_re = re.escape(expression)
-        expr_re = re.sub(r"\\<lang\\>", '<lang>', expr_re)
-        expr_re = re.sub(r"%s?<lang>" % os.sep, '%(sep)s([^%(sep)s]+)'
-            % { 'sep': os.sep}, '.*%s$' % expr_re)
-    print expr_re
-    expr_rec = re.compile(expr_re)
 
     # The path everything will be relative to
-    curpath = os.curdir
+    curpath = os.path.abspath(os.curdir)
+
+    # Force expr to be a valid regex expr (escaped) but keep <lang> intact
+    expr_re = utils.regex_from_filefilter(expression, curpath)
+    expr_rec = re.compile(expr_re)
 
     if not execute:
         utils.MSG("Only printing the commands which will be run if the "
@@ -259,7 +253,7 @@ def _auto_local(path_to_tx, resource, source_language, expression, execute=False
     translation_files = {}
     for root, dirs, files in os.walk(curpath):
         for f in files:
-            f_path = os.path.join(root, f)
+            f_path = os.path.abspath(os.path.join(root, f))
             match = expr_rec.match(f_path)
             if match:
                 lang = match.group(1)
@@ -301,7 +295,10 @@ def _auto_local(path_to_tx, resource, source_language, expression, execute=False
     if execute:
         utils.MSG("Updating file expression for resource %s ( %s )." % (resource,
             expression))
-        prj.config.set("%s" % resource, "file_filter", expression)
+        # Eval file_filter relative to root dir
+        file_filter = relpath(os.path.join(curpath, expression),
+            path_to_tx)
+        prj.config.set("%s" % resource, "file_filter", file_filter)
     else:
         for (lang, f_path) in sorted(translation_files.items()):
             utils.MSG('tx set -r %(res)s -l %(lang)s %(file)s' % {
@@ -323,16 +320,22 @@ def _auto_remote(path_to_tx, url):
 
     if type == 'project':
         utils.MSG("Getting details for project %s" % vars['project'])
-        proj_info = utils.get_project_details(vars['hostname'], username,
-            password, vars['project'])
+        proj_info = utils.get_details('project_details',
+            username, password,
+            hostname = vars['hostname'], project = vars['project'])
         resources = [ '.'.join([vars['project'], r['slug']]) for r in proj_info['resources'] ]
         utils.MSG("%s resources found. Configuring..." % len(resources))
     elif type == 'release':
         utils.MSG("Getting details for release %s" % vars['release'])
-        rel_info = utils.get_release_details(vars['hostname'], username,
-            password, vars['project'], vars['release'])
-        resources = [ '.'.join([r['project_slug'] if r.has_key('project_slug')
-            else vars['project'], r['slug']]) for r in rel_info['resources'] ]
+        rel_info = utils.get_details('release_details',
+            username, password, hostname = vars['hostname'],
+            project = vars['project'], release = vars['release'])
+        resources = []
+        for r in rel_info['resources']:
+            if r.has_key('project_slug'):
+                resources.append(r['project_slug'])
+            else:
+                resources.append('.'.join(vars['project'], r['slug']))           
         utils.MSG("%s resources found. Configuring..." % len(resources))
     elif type == 'resource':
         utils.MSG("Getting details for resource %s" % vars['resource'])
@@ -343,9 +346,9 @@ def _auto_remote(path_to_tx, url):
     for resource in resources:
         utils.MSG("Configuring resource %s." % resource)
         proj, res = resource.split('.')
-        res_info = utils.get_resource_details(vars['hostname'],
-            username, password,
-            proj, res)
+        res_info = utils.get_details('resource_details',
+             username, password, hostname = vars['hostname'],
+             project = proj, resource=res)
         try:
             source_lang = res_info['source_language']['code']
             i18n_type = res_info['i18n_type']
@@ -361,7 +364,6 @@ def _auto_remote(path_to_tx, url):
 
     prj.save()
 
-
 def cmd_push(argv, path_to_tx):
     "Push local files to remote server"
     usage="usage: %prog [tx_options] push [options]"
@@ -375,10 +377,10 @@ def cmd_push(argv, path_to_tx):
         " Transifex but you can filter this per resource or/and language."
     parser = OptionParser(usage=usage, description=description)
     parser.add_option("-l","--language", action="store", dest="languages",
-        default=None, help="Specify which translations you want to pull"
+        default=None, help="Specify which translations you want to push"
         " (defaults to all)")
     parser.add_option("-r","--resource", action="store", dest="resources",
-        default=None, help="Specify the resource for which you want to pull"
+        default=None, help="Specify the resource for which you want to push"
         " the translations (defaults to all)")
     parser.add_option("-f","--force", action="store_true", dest="force_creation",
         default=False, help="Push source files without checking modification"
@@ -398,8 +400,16 @@ def cmd_push(argv, path_to_tx):
 
     force_creation = options.force_creation
 
-    languages = options.languages.split(',') if options.languages else []
-    resources = options.resources.split(',') if options.resources else []
+    if options.languages:
+        languages = options.languages.split(',') 
+    else:
+        languages = []
+
+    if options.resources:
+        resources = options.resources.split(',')
+    else:
+        resources = []
+    
     skip = options.skip_errors
 
     # instantiate the project.Project
@@ -440,8 +450,14 @@ def cmd_pull(argv, path_to_tx):
     parser.add_option("-a","--all", action="store_true", dest="fetchall",
         default=False, help="Fetch all translation files from server (even new"
         " ones)")
+    parser.add_option("-s","--source", action="store_true", dest="fetchsource",
+        default=False, help="Force the fetching of the source file (default:"
+        " False)")
     parser.add_option("-f","--force", action="store_true", dest="force",
         default=False, help="Force download of translations files.")
+    parser.add_option("--skip", action="store_true", dest="skip_errors",
+        default=False, help="Don't stop on errors. Useful when pushing many"
+        " files concurrently.")
     parser.add_option("--disable-overwrite", action="store_false",
         dest="overwrite", default=True,
         help="By default transifex will fetch new translations files and"\
@@ -454,8 +470,17 @@ def cmd_pull(argv, path_to_tx):
         parser.error("You can't user a language filter along with the"\
             " -a|--all option")
 
-    languages = options.languages.split(',') if options.languages else []
-    resources = options.resources.split(',') if options.resources else []
+    if options.languages:
+        languages = options.languages.split(',') 
+    else:
+        languages = []
+
+    if options.resources:
+        resources = options.resources.split(',')
+    else:
+        resources = []
+
+    skip = options.skip_errors
 
     os.chdir(path_to_tx)
     # instantiate the project.Project
@@ -466,8 +491,8 @@ def cmd_pull(argv, path_to_tx):
             raise Exception("Specified resource '%s' does not exist." % r)
 
     prj.pull(languages=languages, resources=resources, overwrite=options.overwrite,
-        fetchall=options.fetchall, force=options.force)
-    prj.save()
+        fetchall=options.fetchall, fetchsource=options.fetchsource,
+        force=options.force, skip=skip)
 
     utils.MSG("Done.")
 
@@ -506,11 +531,12 @@ def _set_source_file(path_to_tx, resource, lang, path_to_file):
 
     # FIXME: Check also if the path to source file already exists.
     try:
-        prj.config.get("%s.%s" % (proj, res), "source_file")
-    except ConfigParser.NoSectionError:
-        prj.config.add_section("%s.%s" % (proj, res))
-    except ConfigParser.NoOptionError:
-        pass
+        try:
+            prj.config.get("%s.%s" % (proj, res), "source_file")
+        except ConfigParser.NoSectionError:
+            prj.config.add_section("%s.%s" % (proj, res))
+        except ConfigParser.NoOptionError:
+            pass
     finally:
         prj.config.set("%s.%s" % (proj, res), "source_file",
            path_to_file)
@@ -532,10 +558,9 @@ def _set_translation(path_to_tx, resource, lang, path_to_file):
     # Chdir to the root dir
     os.chdir(path_to_tx)
 
+    # Warn the user if the file doesn't exist
     if not os.path.exists(path_to_file):
-        raise Exception("tx: File ( %s ) does not exist." %
-             os.path.join(path_to_tx, path_to_file))
-
+        utils.MSG("Warning: File '%s' doesn't exist." % path_to_file)
 
     # instantiate the project.Project
     prj = project.Project(path_to_tx)
@@ -544,16 +569,6 @@ def _set_translation(path_to_tx, resource, lang, path_to_file):
     if root_dir not in os.path.normpath(os.path.abspath(path_to_file)):
         raise Exception("File must be under the project root directory.")
 
-    try:
-        map_object = prj.config.get("%s.%s" % (proj, res), "source_file")
-    except ConfigParser.NoSectionError:
-        map_object = None
-    except ConfigParser.NoOptionError:
-        pass
-
-    if not map_object:
-        raise Exception("tx: You should first run 'set --source' to map the source file.")
-
     if lang ==  prj.config.get("%s.%s" % (proj, res), "source_lang"):
         raise Exception("tx: You cannot set translation file for the source language."
             " Source languages contain the strings which will be translated!")
@@ -577,10 +592,18 @@ def cmd_status(argv, path_to_tx):
         default=[], help="Specify resources")
 
     (options, args) = parser.parse_args(argv)
+    if options.resources:
+        resources = options.resources.split(',')
+    else:
+        resources = []
 
     prj = project.Project(path_to_tx)
-
-    resources = prj.get_resource_list()
+    available_resources = prj.get_resource_list()
+    for r in resources:
+        if not r in available_resources:
+            raise Exception("Specified resource '%s' does not exist." % r)
+    if not resources:
+        resources = available_resources
     resources_num = len(resources)
     for id, res in enumerate(resources):
         p, r = res.split('.')
