@@ -1,11 +1,12 @@
+# -*- coding: utf-8 -*-
 import base64
 import copy
 import getpass
 import os
 import re
 import urllib2
-import ConfigParser
 import datetime, time
+import ConfigParser
 
 from txclib.web import *
 from txclib.utils import *
@@ -60,7 +61,7 @@ class Project(object):
             open(self.txrc_file, 'w').close()
             os.umask(mask)
 
-        self.txrc = ConfigParser.RawConfigParser()
+        self.txrc = OrderedRawConfigParser()
         try:
             self.txrc.read(self.txrc_file)
         except Exception, err:
@@ -114,7 +115,7 @@ class Project(object):
             self.config.add_section(resource)
 
         p_slug, r_slug = resource.split('.')
-        file_filter = re.sub("<sep>", r"%s" % os.sep, file_filter)
+        file_filter = file_filter.replace("<sep>", r"%s" % os.path.sep)
 
         self.config.set(resource, 'source_lang', source_lang)
         self.config.set(resource, 'file_filter', file_filter % {'proj': p_slug,
@@ -148,14 +149,12 @@ class Project(object):
             except ConfigParser.NoOptionError:
                 file_filter = "$^"
             source_lang = self.config.get(resource, "source_lang")
-
-            expr_re = re.escape(file_filter)
-            expr_re = re.sub(r"\\<lang\\>", '<lang>', expr_re)
-            expr_re = re.sub(r"<lang>", '([^/]+)', '.*?%s$' % expr_re)
+            source_file = self.get_resource_option(resource, 'source_file') or None
+            expr_re = regex_from_filefilter(file_filter, self.root)
             expr_rec = re.compile(expr_re)
             for root, dirs, files in os.walk(self.root):
                 for f in files:
-                    f_path = os.path.join(root, f).replace(os.path.sep, '/')
+                    f_path = os.path.abspath(os.path.join(root, f))
                     match = expr_rec.match(f_path)
                     if match:
                         lang = match.group(1)
@@ -166,6 +165,19 @@ class Project(object):
             for (name, value) in self.config.items(resource):
                 if name.startswith("trans."):
                     lang = name.split('.')[1]
+                    # delete language which has same file
+                    if value in tr_files.values():
+                        keys = []
+                        for k, v in tr_files.iteritems():
+                            if v == value:
+                                keys.append(k)
+                        if len(keys) == 1:
+                            del tr_files[keys[0]]
+                        else:
+                            raise Exception("Your configuration seems wrong."\
+                                " You have multiple languages pointing to"\
+                                " the same file.")
+                    # Add language with correct file
                     tr_files.update({lang:value})
 
             return tr_files
@@ -224,7 +236,7 @@ class Project(object):
             return os.path.join(self.root, relpath)
 
     def pull(self, languages=[], resources=[], overwrite=True, fetchall=False,
-        force=False):
+        fetchsource=False, force=False, skip=False):
         """
         Pull all translations file from transifex server
         """
@@ -245,9 +257,6 @@ class Project(object):
                 file_filter = None
 
             # Pull source file
-            MSG("Pulling translations for resource %s (source: %s)" %
-                (resource, sfile))
-
             pull_languages = []
             new_translations = []
 
@@ -270,55 +279,75 @@ class Project(object):
                     MSG("New translations found for the following languages: %s" %
                         ', '.join(new_translations))
 
-            # Check if given language codes exist
             if not languages:
-                pull_languages = files.keys()
+                pull_languages.extend(files.keys())
             else:
                 pull_languages.extend(languages)
                 f_langs = files.keys()
                 for l in languages:
                     if l not in f_langs:
                         pull_languages.remove(l)
-                        ERRMSG("Warning: No mapping found for language code '%s'." %
-                            color_text(l,"RED"))
+                        new_translations.append(l)
+
+            if fetchsource:
+                if sfile and slang not in pull_languages:
+                    pull_languages.append(slang)
+                elif slang not in new_translations:
+                    new_translations.append(slang)
+
+            if pull_languages:
+                MSG("Pulling translations for resource %s (source: %s)" %
+                    (resource, sfile))
 
             for lang in pull_languages:
-                local_file = files[lang]
-
                 if languages and lang not in pull_languages:
                     continue
+                if lang != slang:
+                    local_file = files[lang]
+                else:
+                    local_file = sfile
 
                 if not force:
                     # Check remote timestamp for file and skip update if needed
-                    r = self.do_url_request('resource_stats',
-                        host=host,
-                        project=project_slug,
-                        resource=resource_slug,
-                        language=lang)
+                    try:
+                        r = self.do_url_request('resource_stats',
+                            host=host,
+                            project=project_slug,
+                            resource=resource_slug,
+                            language=lang)
+
+                        stats = parse_json(r)
+                    except Exception,e:
+                        stats = {}
 
-                    stats = parse_json(r)
                     if stats.has_key(lang):
                         time_format = "%Y-%m-%d %H:%M:%S"
 
                         try:
-                            remote_time = time.mktime(datetime.datetime.strptime(stats[lang]['last_update'], time_format).utctimetuple())
-                        except TypeError,e:
+                            remote_time = time.mktime(datetime.datetime(*time.strptime(stats[lang]['last_update'], time_format)[0:5]).utctimetuple())
+                        except (KeyError,TypeError), e:
                             remote_time = None
-                        local_time = time.mktime(time.gmtime(os.path.getmtime(local_file)))
+                        local_time = time.mktime(time.gmtime(os.path.getmtime(self.get_full_path(local_file))))
 
-
-                        if remote_time and remote_time < local_time:
+                        if not remote_time or remote_time and remote_time < local_time:
                             MSG("Skipping '%s' translation (file: %s)." % (color_text(lang, "RED"), local_file))
                             continue
 
                 if not overwrite:
                     local_file = ("%s.new" % local_file)
                 MSG(" -> %s: %s" % (color_text(lang,"RED"), local_file))
-                r = self.do_url_request('pull_file',
-                    host=host,
-                    project=project_slug,
-                    resource=resource_slug,
-                    language=lang)
+                try:
+                    r = self.do_url_request('pull_file',
+                        host=host,
+                        project=project_slug,
+                        resource=resource_slug,
+                        language=lang)
+                except Exception,e:
+                    if not skip:
+                        raise e
+                    else:
+                        ERRMSG(e)
+                        continue
                 base_dir = os.path.split(local_file)[0]
                 mkdir_p(base_dir)
                 fd = open(local_file, 'w')
@@ -326,12 +355,12 @@ class Project(object):
                 fd.close()
 
             if new_translations:
-                MSG("Pulling translations for resource %s (source: %s)" %
+                MSG("Pulling new translations for resource %s (source: %s)" %
                 (resource, sfile))
                 for lang in new_translations:
                     if file_filter:
                         local_file = relpath(os.path.join(self.root,
-                            re.sub('<lang>', lang, file_filter)), os.curdir)
+                            file_filter.replace('<lang>', lang)), os.curdir)
                     else:
                         trans_dir = os.path.join(self.root, ".tx", resource)
                         if not os.path.exists(trans_dir):
@@ -352,7 +381,6 @@ class Project(object):
                     fd.write(r)
                     fd.close()
 
-
     def push(self, source=False, translations=False, force=False, resources=[], languages=[],
         skip=False, no_interactive=False):
         """
@@ -364,6 +392,7 @@ class Project(object):
             resource_list = self.get_resource_list()
 
         for resource in resource_list:
+            push_languages = []
             project_slug, resource_slug = resource.split('.')
             files = self.get_resource_files(resource)
             slang = self.get_resource_option(resource, 'source_lang')
@@ -426,20 +455,19 @@ class Project(object):
             if translations:
                 # Check if given language codes exist
                 if not languages:
-                    languages = files.keys()
+                    push_languages = files.keys()
                 else:
+                    push_languages = list(languages)
                     f_langs = files.keys()
-                    for l in languages:
+                    for l in push_languages:
                         if l not in f_langs:
-                            languages.remove(l)
+                            push_languages.remove(l)
                             ERRMSG("Warning: No mapping found for language code '%s'." %
                                 color_text(l,"RED"))
 
                 # Push translation files one by one
-                for lang in languages:
+                for lang in push_languages:
                     local_file = files[lang]
-                    if languages and lang not in languages:
-                        continue
 
                     if not force:
                         try:
@@ -452,10 +480,10 @@ class Project(object):
                             # Check remote timestamp for file and skip update if needed
                             stats = parse_json(r)
                             time_format = "%Y-%m-%d %H:%M:%S"
-                            remote_time = time.mktime(datetime.datetime.strptime(stats[lang]['last_update'], time_format).utctimetuple())
+                            remote_time = time.mktime(datetime.datetime(*time.strptime(stats[lang]['last_update'], time_format)[0:5]).utctimetuple())
                         except Exception, e:
                             remote_time = None
-                        local_time = time.mktime(time.gmtime(os.path.getmtime(local_file)))
+                        local_time = time.mktime(time.gmtime(os.path.getmtime(self.get_full_path(local_file))))
 
                         if remote_time and remote_time > local_time:
                             MSG("Skipping '%s' translation (file: %s)." % (color_text(lang, "RED"), local_file))
@@ -491,7 +519,6 @@ class Project(object):
         Issues a url request.
         """
         # Read the credentials from the config file (.transifexrc)
-        
         try:
             username = self.txrc.get(host, 'username')
             passwd = self.txrc.get(host, 'password')
@@ -506,10 +533,6 @@ class Project(object):
         kwargs['hostname'] = hostname
         url = API_URLS[api_call] % kwargs
 
-        password_manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
-        password_manager.add_password(None, hostname, username, passwd)
-        auth_handler = urllib2.HTTPBasicAuthHandler(password_manager)
-
         opener = None
         headers = None
         req = None
@@ -524,8 +547,6 @@ class Project(object):
             urllib2.install_opener(opener)
             req = RequestWithMethod(url=url, data=data, method=method)
         else:
-            opener = urllib2.build_opener(auth_handler)
-            urllib2.install_opener(opener)
             req = RequestWithMethod(url=url, data=data, method=method)
             if encoding:
                 req.add_header("Content-Type",encoding)
